@@ -2,7 +2,7 @@ package Net::DNS::RR::SVCB;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: SVCB.pm 1993 2024-11-07 14:06:53Z willem $)[2];
+our $VERSION = (qw$Id: SVCB.pm 2037 2025-08-18 14:39:32Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -47,10 +47,8 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $params = $self->{SvcParams} = [];
 	while ( ( my $start = $offset + 4 ) <= $limit ) {
 		my ( $key, $size ) = unpack( "\@$offset n2", $rdata );
-		my $next = $start + $size;
-		last if $next > $limit;
+		last if ( $offset = $start + $size ) > $limit;
 		push @$params, ( $key, substr $rdata, $start, $size );
-		$offset = $next;
 	}
 	die $self->type . ': corrupt RDATA' unless $offset == $limit;
 	return;
@@ -82,7 +80,7 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $length = 2 + length $encode;
 	my @target = grep {length} split /(\S{32})/, unpack 'H*', $encode;
 	my $target = substr $self->{TargetName}->string, 0, 40;
-	push @rdata, join '', shift(@target), "\t; $target\n";
+	push @rdata, join '', shift(@target), "\t; $target\n" unless $target eq '.';
 	push @rdata, @target;
 
 	my $params = $self->{SvcParams} || [];
@@ -118,7 +116,9 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 	while ( my $svcparam = shift @argument ) {
 		for ($svcparam) {
 			my @value;
-			if (/^key\d+=(.*)$/i) {
+			if (/^key\d+$/i) {
+				push @value, '';
+			} elsif (/^key\d+=(.*)$/i) {
 				local $_ = length($1) ? $1 : shift @argument;
 				s/^"([^"]*)"$/$1/;		# strip enclosing quotes
 				push @value, $_;
@@ -126,17 +126,14 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 				local $_ = length($1) ? $1 : shift @argument;
 				die <<"Amen" if /\\092[,\\]/;
 SVCB:	Please use standard RFC1035 escapes
-	RFC9460 double-escape nonsense not implemented
+	RFC9460 double-escape insanity not implemented
 Amen
 				s/^"([^"]*)"$/$1/;		# strip enclosing quotes
 				s/\\,/\\044/g;			# disguise (RFC1035) escaped comma
 				push @value, split /,/;
-			} else {
-				push @value, '' unless $keybyname{lc $_};    # empty | Boolean
 			}
 
-			s/[-]/_/g;				# extract identifier
-			m/^([^=]+)/;
+			m/^([^=]+)/;				# extract identifier
 			$self->$1(@value);
 		}
 	}
@@ -176,18 +173,16 @@ sub _defaults {				## specify RR attribute default values
 sub svcpriority {
 	my ( $self, @value ) = @_;				# uncoverable pod
 	for (@value) { $self->{SvcPriority} = 0 + $_ }
-	return $self->{SvcPriority} || 0;
+	return $self->{SvcPriority};
 }
 
 
 sub targetname {
 	my ( $self, @value ) = @_;				# uncoverable pod
-
-	for (@value) { $self->{TargetName} = Net::DNS::DomainName->new($_) }
-
+	$self->{TargetName} = Net::DNS::DomainName->new(@value) if @value;
 	my $target = $self->{TargetName} ? $self->{TargetName}->name : return;
-	return $target unless $self->{SvcPriority};
-	return ( $target eq '.' ) ? $self->owner : $target;
+	return $target unless $target eq '.';
+	return $self->{SvcPriority} ? $self->owner : undef;
 }
 
 
@@ -195,92 +190,98 @@ sub mandatory {				## mandatory=key1,port,...
 	my ( $self, @value ) = @_;
 	my @list = map { $keybyname{lc $_} || $_ } map { split /,/ } @value;
 	my @keys = map { /(\d+)$/ ? $1 : die( $self->type . qq[: unexpected "$_"] ) } @list;
-	return $self->key0( _integer16( sort { $a <=> $b } @keys ) );
+	return $self->_SvcParam( 0, _integer16( sort { $a <=> $b } @keys ) );
 }
 
 sub alpn {				## alpn=h3,h2,...
 	my ( $self, @value ) = @_;
-	return $self->key1( _string(@value) );
+	return $self->_SvcParam( 1, _string(@value) );
 }
 
 sub no_default_alpn {			## no-default-alpn	(Boolean)
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key2( defined(wantarray) ? () : _boolean(@value) );
+	return $self->_SvcParam(2) if defined wantarray;
+	return $self->_SvcParam( 2, _boolean(@value) );
 }
 
 sub port {				## port=1234
 	my ( $self, @value ) = @_;
-	return $self->key3( map { _integer16($_) } @value );
+	return $self->_SvcParam( 3, map { _integer16($_) } @value );
 }
 
 sub ipv4hint {				## ipv4hint=192.0.2.1,...
 	my ( $self, @value ) = @_;
-	return $self->key4( _ipv4(@value) );
+	return $self->_SvcParam( 4, _address4(@value) );
 }
 
 sub ech {				## ech=base64
 	my ( $self, @value ) = @_;
-	return $self->key5( map { _base64($_) } @value );
+	return $self->_SvcParam( 5, map { _base64($_) } @value );
 }
 
 sub ipv6hint {				## ipv6hint=2001:DB8::1,...
 	my ( $self, @value ) = @_;
-	return $self->key6( _ipv6(@value) );
+	return $self->_SvcParam( 6, _address6(@value) );
 }
 
 sub dohpath {				## dohpath=/dns-query{?dns}
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key7(@value);
+	return $self->_SvcParam( 7, map { substr _string($_), 1 } @value );
 }
 
 sub ohttp {				## ohttp
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key8( defined(wantarray) ? () : _boolean(@value) );
+	return $self->_SvcParam(8) if defined wantarray;
+	return $self->_SvcParam( 8, _boolean(@value) );
 }
 
 sub tls_supported_groups {		## tls_supported_groups=29,23
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key9( _integer16(@value) );
+	return $self->_SvcParam( 9, _integer16(@value) );
 }
 
 
 ########################################
 
 
-sub _presentation {			## represent octet string(s) using local charset
+sub _concatenate {			## concatenate octet string(s)
 	my @arg = @_;
-	my $raw = scalar(@arg) ? join( '', @arg ) : return ();	# concatenate arguments
-	return Net::DNS::Text->decode( \$raw, 0, length($raw) )->string;
+	return scalar(@arg) ? join( '', @arg ) : @arg;
 }
 
 sub _boolean {
 	my @arg = @_;
-	return ( '', @arg );
+	return '' unless scalar @arg;
+	return map { $_ ? '' : undef } @arg;
 }
 
 sub _string {
 	my @arg = @_;
-	return _presentation( map { Net::DNS::Text->new($_)->encode() } @arg );
+	my @val = map { split /,/ } @arg;
+	return _concatenate( map { Net::DNS::Text->new($_)->encode() } @val );
 }
 
 sub _base64 {
 	my @arg = @_;
-	return _presentation( map { MIME::Base64::decode($_) } @arg );
+	return _concatenate( map { MIME::Base64::decode($_) } @arg );
 }
 
 sub _integer16 {
 	my @arg = @_;
-	return _presentation( map { pack( 'n', $_ ) } @arg );
+	my @val = map { split /,/ } @arg;
+	return _concatenate( map { pack( 'n', $_ ) } @val );
 }
 
-sub _ipv4 {
+sub _address4 {
 	my @arg = @_;
-	return _presentation( map { Net::DNS::RR::A::address( {}, $_ ) } @arg );
+	my @val = map { split /,/ } @arg;
+	return _concatenate( map { Net::DNS::RR::A::address( {}, $_ ) } @val );
 }
 
-sub _ipv6 {
+sub _address6 {
 	my @arg = @_;
-	return _presentation( map { Net::DNS::RR::AAAA::address( {}, $_ ) } @arg );
+	my @val = map { split /,/ } @arg;
+	return _concatenate( map { Net::DNS::RR::AAAA::address( {}, $_ ) } @val );
 }
 
 
@@ -288,12 +289,29 @@ sub AUTOLOAD {				## Dynamic constructor/accessor methods
 	my ( $self, @argument ) = @_;
 
 	our $AUTOLOAD;
-	my ($method) = reverse split /::/, $AUTOLOAD;
+	my ($method)  = reverse split /::/, $AUTOLOAD;
+	my $canonical = lc($method);
+	$canonical =~ s/-/_/g;
+	if ( $self->can($canonical) ) {
+		no strict 'refs';	## no critic ProhibitNoStrict
+		*{$AUTOLOAD} = sub { shift->$canonical(@_) };
+		return $self->$canonical(@argument);
+	}
 
 	my $super = "SUPER::$method";
 	return $self->$super(@argument) unless $method =~ /^key[0]*(\d+)$/i;
 	my $key = $1;
 
+	return $self->_SvcParam($key) unless @argument;
+	my $first = shift @argument;
+	my $value = defined $first ? Net::DNS::Text->new($first)->raw : $first;
+	return $self->_SvcParam( $key, $value, @argument );
+}
+
+
+sub _SvcParam {
+	my ( $self, $key, @argument ) = @_;
+
 	my $paramsref = $self->{SvcParams} || [];
 	my %svcparams = @$paramsref;
 
@@ -304,14 +322,13 @@ sub AUTOLOAD {				## Dynamic constructor/accessor methods
 		die( $self->type . qq[: invalid SvcParam "key$key"] )	if $key > 65534;
 		die( $self->type . qq[: unexpected "key$key" value] )	if scalar @argument;
 		delete $self->{rdata};
-		$svcparams{$key} = Net::DNS::Text->new("$arg")->raw if defined $arg;
+		$svcparams{$key} = $arg if defined $arg;
 		$self->{SvcParams} = [map { ( $_, $svcparams{$_} ) } sort { $a <=> $b } keys %svcparams];
 	} else {
 		die( $self->type . qq[: no value specified for "key$key"] ) unless defined wantarray;
 	}
 
-	my $value = $svcparams{$key};
-	return defined($value) ? _presentation($value) : $value;
+	return $svcparams{$key};
 }
 
 ########################################
@@ -323,8 +340,8 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name HTTPS SvcPriority TargetName SvcParams');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name SVCB  SvcPriority TargetName SvcParams');
 
 =head1 DESCRIPTION
 
@@ -345,17 +362,17 @@ other unpredictable behaviour.
 
 =head2 SvcPriority
 
-    $svcpriority = $rr->svcpriority;
-    $rr->svcpriority( $svcpriority );
+	$rr->SvcPriority( $svcpriority );
+	$svcpriority = $rr->SvcPriority;
 
-The priority of this record
-(relative to others, with lower values preferred). 
-A value of 0 indicates AliasMode.
+The priority of this record relative to others in the RRset,
+(lower values being preferred). 
+A value of 0 indicates AliasMode.')
 
 =head2 TargetName
 
-    $rr->targetname( $targetname );
-    $effecivetarget = $rr->targetname;
+	$rr->targetname( $targetname );
+	$effecivetarget = $rr->targetname;
 
 The domain name of either the alias target (for AliasMode)
 or the alternative endpoint (for ServiceMode).
@@ -368,33 +385,34 @@ owner name of this record must be used as the effective TargetName.
 
 =head2 mandatory, alpn, no-default-alpn, port, ipv4hint, ech, ipv6hint
 
-    $rr = Net::DNS::RR->new( 'svcb.example. SVCB 1 svcb.example. port=1234' );
+	$rr = Net::DNS::RR->new( 'svcb.example. SVCB 1 svcb.example. port=1234' );
 
-    $rr->port(1234);
-    $string = $rr->port();	# \004\210
-    $rr->key3($string);
+	$rr->port(1234);
+	$octets = $rr->port();	# 0x04 0xD2
+	$octets = $rr->key3();
 
 Constructor methods for mnemonic SvcParams prescribed by RFC9460.
-When invoked without arguments, the methods return the presentation format
-value for the underlying key.
+When invoked without arguments, the methods return the value
+of the underlying key as an uninterpreted octet string.
 The behaviour with undefined arguments is not specified.
 
 =head2 keyNN
 
-    $keynn = $rr->keyNN;
-    $rr->keyNN( $keynn );
-    $rr->keyNN( undef );
+	$keynn = $rr->keyNN;
+	$rr->keyNN( $keynn );
+	$rr->keyNN( undef );
 
 Generic constructor and accessor methods for SvcParams.
 The key index NN is a decimal integer in the range 0 .. 65535.
-The method argument and returned value are both presentation format strings.
+The method argument is a presentation format character string.
+The returned value is an uninterpreted octet string.
 The method returns the undefined value if the key is not present.
-The specified key will be deleted if the value is undefined.
+The specified key will be deleted if the argument is undefined.
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2020-2022 Dick Franks. 
+Copyright (c)2020-2025 Dick Franks. 
 
 All rights reserved.
 
